########################################################
###                                                  ###
###   Multidimensional combinatorial sustitutions    ###
###                                                  ###
###   Timo Jolivet, 07/06/2012                       ###
###                                                  ###
###   This is work in progress to be tidied up.      ###
###   Use at your own risk!                          ###
###                                                  ###
########################################################

import itertools

class Cell(SageObject):
    r"""
    Cells of Z^d patterns.

    EXAMPLES:

        sage: c = Cell([0,1],2)
        sage: c
        [(0, 1), 2]
        sage: c.vector()
        (0, 1)
        sage: c.type()
        2
        sage: c.color()
        RGB color (1.0, 1.0, 0.0)

    """
    def __init__(self, v, t, color=None):
        r"""
        """
        self._vector = (ZZ**(len(v)))(v)
        self._vector.set_immutable()
        self._type = t

        if color is None:
            if t == 1:
                color = "red"
            elif t == 2:
                color = "yellow"
            elif t == 3:
                color = "green"
            elif t == 4:
                color = "purple"
            elif t == 5:
                color = "orange"
            elif t == 6:
                color = "blue"
            elif t == 7:
                color = "pink"
            elif t == 8:
                color = "white"
            else:
                color = (0.3, 0.3, 0.3)

        self._color = Color(color)

        r"""
        """

    def __repr__(self):
        r"""
        """
        return "[%s, %s]"%(self.vector(), self.type())

    def __eq__(self, other):
        r"""
        """
        return (isinstance(other, Cell) and
                self.vector() == other.vector() and
                self.type() == other.type())

    def __hash__(self):
        r"""
        """
        return hash((self.vector(), self.type()))

    def _boundary(self):
        r"""
        """
        try:
            x, y = self.vector()
        except ValueError:
            raise NotImplementedError, "Boundary is implemented only for two-dimensional patterns."

        e1 = (x,y,x+1,y)
        e2 = (x+1,y,x+1,y+1)
        e3 = (x,y+1,x+1,y+1)
        e4 = (x,y,x,y+1)

        return FormalSum([(1,e1), (1,e2), (-1,e3), (-1,e4)])

    def _plot(self, opacity, print_text, size3d=1):
        r"""
        """
        v = self.vector()
        t = self.type()
        c = self.color()

        if len(v) == 1:
            G = polygon2d([vector([v[0] + u[0], u[1]]) for u in [[0,0], [0,0.25], [1,0.25], [1,0]]], rgbcolor=c, alpha=opacity, thickness=1)
            G += line2d([vector([v[0] + u[0], u[1]]) for u in [[0,0], [0,0.25], [1,0.25], [1,0]]], rgbcolor=(0,0,0))
            if print_text:
                G += text(str(t), vector([v[0] + 0.5,0.5]), rgbcolor=(0,0,0))
            return G

        elif len(v) == 2:
            G = polygon2d([v + vector(u) for u in [[0,0], [0,1], [1,1], [1,0]]], rgbcolor=c, alpha=opacity, thickness=1)
            G += line2d([v + vector(u) for u in [[0,0], [0,1], [1,1], [1,0], [0,0]]], rgbcolor=(0,0,0))
            if print_text:
                G += text(str(t), v + vector([0.5,0.5]), rgbcolor=(0,0,0))
            return G

        elif len(v) == 3:
            G = cube(v, size=size3d, rgbcolor = c, opacity=opacity)
            return G

        else:
            raise ValueError, 'Plotting implemented only for dimensions 1, 2 and 3.'

    def vector(self):
        r"""
        Returns the vector of ``self``.

        EXAMPLES:

            sage: c = Cell([0,1],2)
            sage: c.vector()
            (0, 1)
        """
        return self._vector

    def type(self):
        r"""
        Returns the type of ``self``.

        EXAMPLES:

            sage: c = Cell([0,1],2)
            sage: c.type()
            2
        """
        return self._type

    def color(self, color=None):
        r"""
        Returns the color of ``self`` if `color` is None.
        Sets the color of ``self`` to ``color`` otherwise.

        EXAMPLES:

            sage: c = Cell([0,1],2)
            sage: c.color()
            RGB color (1.0, 1.0, 0.0)
            sage: c.color("pink")
            sage: c.color()
            RGB color (1.0, 0.7529411764705882, 0.796078431372549)
        """
        if color is None:
            return self._color
        else:
            self._color = Color(color)

class Pattern(SageObject):
    r"""
    Patterns on Z^d.

    EXAMPLES:

    #. Defining a pattern using a list of cells::

        sage: P = Pattern([Cell([0,0],1), Cell([1,0],2), Cell([0,1],3)])
        sage: P
        Pattern: [[(0, 0), 1], [(1, 0), 2], [(0, 1), 3]]

    #. Defining a pattern using a matrix (zeroes are interpreted as empty spaces)::

        sage: M = matrix(3, [1,0,0,2,1,3,0,4,5])
        sage: M
        [1 0 0]
        [2 1 3]
        [0 4 5]
        sage: P = Pattern(M)
        sage: P
        Pattern: [[(1, 0), 4], [(2, 0), 5], [(0, 1), 2], [(1, 1), 1], [(2, 1), 3], [(0, 2), 1]]
    """
    def __init__(self, X):
        r"""
        """
        # quick way to define patterns with matrices, where 0 means "empty" and the cell types are integers
        if isinstance(X, sage.matrix.matrix_integer_dense.Matrix_integer_dense):
            L = []
            for i in range(X.nrows()):
                for j in range(X.ncols()):
                    if X[-i-1][j] != 0:
                        L.append(Cell([j,i], X[-i-1][j]))
            self._cells = list(L)

        else:
            self._cells = [Cell(c.vector(), c.type(), c.color()) for c in X]

    def __eq__(self, other):
        r"""
        """
        return (isinstance(other, Pattern) and set(self) == set(other))

    def __len__(self):
        r"""
        """
        return len(self._cells)

    def __repr__(self):
        r"""
        """
        if len(self) <= 20:
            return 'Pattern: %s'%self._cells
        else:
            return 'Pattern of %s cell(s)'%len(self)

    def __iter__(self):
        r"""
        """
        return iter(self._cells)

    def __getitem__(self, key):
        r"""
        """
        return self._cells[key]

    def __add__(self, other):
        r"""
        See ``self.union`` method.
        """
        return self.union(other)

    def __sub__(self, other):
        r"""
        See ``self.difference`` method.
        """
        return self.difference(other)

    def union(self, other):
        r"""
        Returns the union of ``self`` and ``other``.

        EXAMPLES:

            sage: P = Pattern([Cell([0,0],1), Cell([1,0],2), Cell([0,1],3)])
            sage: Q = Pattern([Cell([1,1],1), Cell([2,1],3)])
            sage: f = Cell([-1,-1],1)
            sage: P.union(Q)
            Pattern: [[(0, 0), 1], [(1, 0), 2], [(0, 1), 3], [(1, 1), 1], [(2, 1), 3]]
            sage: P.union(f)
            Pattern: [[(0, 0), 1], [(1, 0), 2], [(0, 1), 3], [(-1, -1), 1]]
        """
        if isinstance(other, Cell):
            return Pattern(self._cells + [other])
        else:
            return Pattern(self._cells + other._cells)

    def difference(self, other):
        r"""
        Returns the difference of ``self`` and ``other``.

        EXAMPLES:

            sage: P = Pattern([Cell([0,0],1), Cell([1,0],2), Cell([0,1],3)])
            sage: Q = Pattern([Cell([0,0],1), Cell([0,1],3)])
            sage: f = Cell([0,1],3)
            sage: P - Q
            Pattern: [[(1, 0), 2]]
            sage: P - f
            Pattern: [[(0, 0), 1], [(1, 0), 2]]
        """
        if isinstance(other, Cell):
            return Pattern(set(self._cells) - set([other]))
        else:
            return Pattern(set(self._cells) - set(other._cells))

    def cells_of_vector(self, v):
        r"""
        Returns the cells of ``self`` that have vector ``v``.

        EXAMPLES:

            sage: P = Pattern([Cell([0,0],1), Cell([1,0],2), Cell([0,1],3)])
            sage: P.cells_of_vector([1,0])
            [[(1, 0), 2]]
        """
        return [Cell(c.vector(), c.type(), c.color()) for c in self if c.vector() == vector(v)]

    def cells_of_type(self, t):
        r"""
        Returns the cells of ``self`` that have type ``t``.

        EXAMPLES:

            sage: P = Pattern([Cell([0,0],1), Cell([1,0],2), Cell([0,1],3)])
            sage: P.cells_of_type(2)
            [[(1, 0), 2]]
        """
        return [Cell(c.vector(), c.type(), c.color()) for c in self if c.type() == t]

    def cells_of_color(self, col):
        r"""
        Returns the cells of ``self`` that have color ``col``.

        EXAMPLES:

            sage: P = Pattern([Cell([0,0],1), Cell([1,0],2), Cell([0,1],3)])
            sage: P.cells_of_color([1,0,0])
            [[(0, 0), 1]]
        """
        return [Cell(c.vector(), c.type(), c.color()) for c in self if list(c.color()) == list(col)]

    def support(self):
        r"""
        Returns the set of vectors of cells of ``self``.

        EXAMPLES:

            sage: P = Pattern([Cell([0,0],1), Cell([1,0],2), Cell([0,1],3)])
            sage: P.support()
            [(0, 0), (1, 0), (0, 1)]
        """
        return [cell.vector() for cell in self]

    def translate(self, v):
        r"""
        Returns a copy of ``self`` translated by ``v``.

        EXAMPLES:

            sage: P = Pattern([Cell([0,0],1), Cell([1,0],2), Cell([0,1],3)])
            sage: P.translate([10,20])
            Pattern: [[(10, 20), 1], [(11, 20), 2], [(10, 21), 3]]
        """
        v = vector(v)
        return Pattern([Cell(cell.vector() + v, cell.type(), cell.color()) for cell in self])

    def occurrences_of(self, other):
        r"""
        Returns all positions at which ``other`` appears in ``self``, that is,
        all vectors ``v`` such that ``set(other.translate(v)) <= set(self)``.

        EXAMPLES:

            sage: M = matrix(8, [0,1,2,3]*20)
            sage: M
            [0 1 2 3 0 1 2 3 0 1]
            [2 3 0 1 2 3 0 1 2 3]
            [0 1 2 3 0 1 2 3 0 1]
            [2 3 0 1 2 3 0 1 2 3]
            [0 1 2 3 0 1 2 3 0 1]
            [2 3 0 1 2 3 0 1 2 3]
            [0 1 2 3 0 1 2 3 0 1]
            [2 3 0 1 2 3 0 1 2 3]
            sage: N = matrix(3, [3,0,1,1,2,3,3,0,1])
            sage: N
            [3 0 1]
            [1 2 3]
            [3 0 1]
            sage: P = Pattern(M)
            sage: Q = Pattern(N)
            sage: P.occurrences_of(Q)
            [(1, 0), (5, 0), (3, 1), (7, 1), (1, 2), (5, 2), (3, 3), (7, 3), (1, 4), (5, 4), (3, 5), (7, 5)]
        """
        x = other[0].vector()
        t = other[0].type()
        L = self.cells_of_type(t)
        positions = []
        for cell in L:
            y = cell.vector()
            if set(other.translate(y-x)) <= set(self):
                positions.append(y-x)
        return positions

    def covering_graph(self,
                       L,
                       tree=False,
                       compute_pos=False,
                       plot=False,
                       plot_tikz=False,
                       show=False):
        r"""
        Returns a graph whose vertices are the cells of ``self``.
        Two cells are connected by an edge if and only if
        the two cells appear in a same pattern of ``L``.

        INPUT:

        - ``L`` - a list of patterns

        - ``tree`` - bool (optional, default: ``False``)
          If True, a minimal spanning tree of the graph is computed instead.
          (Or a forest if the graph is not connected.)

        - ``compute_pos`` - bool (optional, default: ``False``)
          If True, the position of each vertex is encoded in the graph,
          according to the pattern Q.

        - ``plot`` - bool (optional, default: ``False``)
          If True, a Graphics object depicting the covering graph is returned.

        - ``plot_tikz`` - bool (optional, default: ``False``)
          If True, Tikz code depicting the covering graph is returned.

        - ``show`` - bool (optional, default: ``False``)
          Same as ``plot``, except that the Graphics object is shown, using
          the ``show`` method, without coordinate axes and with appropriate size.
          (The underlying pattern is shown.)
        """
        G = Graph()
        G.add_vertices(self)
        for P in L:
            for v in self.occurrences_of(P):
                G.add_path(P.translate(v))

        if tree:
            T = []
            for C in G.connected_components():
                T += G.subgraph(C).min_spanning_tree()
            H = Graph()
            H.add_edges(T)
            G = H

        if compute_pos or plot or show:
            D = dict([(cell, cell.vector() + vector(len(cell.vector())*[0.5])) for cell in self])
            G.set_pos(D)

        if plot:
            return G.plot(vertex_labels=False, vertex_size=10)

        elif plot_tikz:
            s = self.plot_tikz() + '\n'
            for (c1, c2, _) in G.edges():
                x1, y1 = c1.vector()
                x2, y2 = c2.vector()
                s += '\\draw[very thick, draw=black!35, cap=round] (%f, %f) -- (%f, %f);\n'%(x1+0.5, y1+0.5, x2+0.5, y2+0.5)
            return s

        elif show:
            return (self.plot() + G.plot(vertex_labels=False, vertex_size=5)).show(figsize=(len(self))^(5/12), axes=False)

        else:
            return G

    def boundary(self, sum=False, ignore_overlaps=False):
        r"""
        Return a list of lists of 2D points.
        To each list corresponds a connected component of the boundary of ``self``.

        Only for 2D patterns.

        INPUT:

        - ``sum`` - bool (optional, default: ``False``)
          Returns a formal sum of the form (x1, y1, x2, y2)
          for each unit edge from (x1,y1) to (x2,y2) in the boundary.

        - ``ignore_overlaps`` - bool (optional, default: ``False``)
          Filter the pattern first to remove overlapping cells
          before computing the boundary.

        EXAMPLES:

        #. Computing a simple "L"-shaped boundary:

            sage: P = Pattern([Cell([0,0],1), Cell([1,0],2), Cell([0,1],3)])
            sage: P.boundary()
            [[(0, 1), (0, 0), (1, 0), (2, 0), (2, 1), (1, 1), (1, 2), (0, 2), (0, 1)]]

        #. Computing a disconnected boundary:
            sage: Q = Pattern([Cell([1,1],1), Cell([3,1],3)])
            sage: Q.boundary()
            [[(1, 2), (1, 1), (2, 1), (2, 2), (1, 2)], [(4, 2), (3, 2), (3, 1), (4, 1), (4, 2)]]

        #. The formal sum associated with a boundary:

            sage: R = Pattern([Cell([7,12],1)])
            sage: R.boundary(sum=True)
            -(7, 12, 7, 13) + (7, 12, 8, 12) - (7, 13, 8, 13) + (8, 12, 8, 13)
        """

        # S: formal sum of boundary edges
        S = 0
        if ignore_overlaps:
            for v in set(self.support()):
                S += Cell(v,1)._boundary()
        else:
            for c in self:
                S += c._boundary()

        if sum:
            return S

        # L: list of vectors (coordinates) of boundary
        L = []

        for i, x in list(S):
            i1, j1, i2, j2 = x
            if i == 1:
                L.append(((i1,j1),(i2,j2)))
            elif i == -1:
                L.append(((i2,j2),(i1,j1)))
            else:
                raise ValueError, 'Something is wrong with the pattern (probably overlapping cells).'

        # G: DiGraph with elements of L as edges
        # cycles = eulerian circuits of the connected components of G
        G = DiGraph()
        for x,y in L:
            G.add_edge(x,y)

        cycles = []
        for H in G.connected_components_subgraphs():
            path = H.eulerian_circuit(labels=false)
            path = [x[0] for x in path] + [path[0][0]]
            cycles.append(path)

        return cycles

    def number_of_dominoes(self):
        r"""
        Returns the number of dominoes contained in ``self``.

        EXAMPLES:

            sage: P = Pattern([Cell([0,0],1), Cell([1,0],2), Cell([1,1],3)])
            sage: P.number_of_dominoes()
            2
        """
        b = sum([len(x) for x in self.boundary()])

        return 2*len(self) - (b-1)/2

    def repaint(self, cmap={}):
        r"""
        Repaints all the cell of ``self`` from the given color map ``cmap``.

        This changes ``self``.

        INPUT:

        -  ``cmap`` - color map (default: ``'prism'``). It can be one of the
           following :

           - string - A coloring map. For available coloring map names type:
             ``sorted(colormaps)``

           - list - a list of colors to assign cyclically to the cells.
             A list of a single color colors all the cells with the same color.

           - dict - a dict of face types mapped to colors, to color the
             cells according to their type.

           - ``{}``, the empty dict - shorcut for
             ``{1:'red', 2:'green', 3:'blue'}``.

        EXAMPLES:

        """
        if cmap == {}:
            cmap = {1: 'red', 2: 'yellow', 3: 'green', 4: 'purple', 5: 'orange', 6: 'blue', 7: 'pink', 8: 'white'}

        if isinstance(cmap, dict):
            for cell in self:
                cell.color(cmap[cell.type()])

        elif isinstance(cmap, list):
            k = len(cmap)
            for i, cell in enumerate(self):
                cell.color(cmap[i%k])

        elif isinstance(cmap, str):
            from matplotlib import cm
            if not cmap in cm.datad.keys():
                raise RuntimeError("Color map %s not known (type sorted(colors) for valid names)" % cmap)
            cmap = cm.__dict__[cmap]
            dim = float(len(self))
            for i, cell in enumerate(self):
                cell.color(cmap(i/dim)[:3])

        else:
            raise TypeError, "Type of cmap (=%s) must be dict, list or str" %cmap

    def plot(self, opacity=1, print_text=True, size3d=1):
        r"""
        """

        if len(self[0].vector()) in [1,2]:
            G = Graphics()
            for cell in self:
                G += cell._plot(opacity, print_text)
            G.set_aspect_ratio(1)
            return G

        elif len(self[0].vector()) == 3:
            G = Graphics()
            for cell in self:
                G += cell._plot(opacity, print_text, size3d)
            return G

        else:
            raise ValueError, 'Plotting implemented only for dimensions 1, 2 and 3.'

    def plot_tikz(self, print_text=True):
        r"""
        """
        if len(self[0].vector()) != 2:
            raise ValueError, 'Tikz plotting implemented only for dimension 2.'

        s = '\\begin{tikzpicture}[x=2.5mm, y=2.5mm]\n'
        color = ()

        for cell in self:
            t = cell.type()
            x, y = cell.vector()

            if tuple(color) != tuple(cell.color()):
                color = cell.color()
                s += '\\definecolor{cellcolor}{rgb}{%.3f,%.3f,%.3f}\n'%(color[0], color[1], color[2])

            s += '\\fill[fill=cellcolor, draw=%s]\n'%('black')
            s += '(%d,%d) -- (%d,%d) -- (%d,%d) -- (%d,%d) -- cycle;\n'%(x, y, x+1, y, x+1, y+1, x, y+1)

            if print_text:
                s += '\\node at (%.1f,%.1f) {\\scriptsize $%s$};\n'%(x+0.5, y+0.5, t)

        s += '\\end{tikzpicture}'

        return s

    def plot_tikz_matrix(self):
        r"""
        """
        if len(self[0].vector()) != 2:
            raise ValueError, 'Matrix plotting implemented only for dimension 2.'

        s = '\\begin{smallmatrix}\n'
        Lh = [v.vector()[0] for v in self]
        Lv = [v.vector()[1] for v in self]
        xmin, xmax = min(Lh), max(Lh)
        ymin, ymax = min(Lv), max(Lv)

        for y in reversed(range(ymin, ymax+1)):
            for x in range(xmin, xmax+1):
                C = self.cells_of_vector(vector((x,y)))
                if x < xmax:
                    line_break = '&'
                else:
                    line_break = '\\\\\n'
                if C:
                    s += ' %d %s'%(C[0].type(), line_break)
                elif not C:
                    s += '   %s'%line_break

        s += '\\end{smallmatrix}'

        return s

    def plot_ascii(self):
        r"""
        """
        tcol = lambda i : "\033[1;%dm"%(30+(i%8))

        if len(self[0].vector()) == 1:
            supp = self.support()
            xmin = supp[0][0]
            xmax = supp[0][0]

            # Compute the bounding box of self in Z
            for x in supp:
                if xmin > x[0]:
                    xmin = x[0]
                elif xmax < x[0]:
                    xmax = x[0]

            strarray = [' ']*(xmax-xmin+1)

            for c in self:
                x = c.vector()[0]
                i = c.type()
                strarray[x-xmin] = tcol(i) + hex(i)

            print "\n",
            print ''.join(strarray)
            print "\033[0m\n",

        elif len(self[0].vector()) == 2:
            supp = self.support()
            v0 = supp[0]
            xmin = v0[0]
            ymin = v0[1]
            xmax = v0[0]
            ymax = v0[1]

            # Compute the bounding box of self in Z^2
            for x,y in supp:
                if xmin > x:
                    xmin = x
                elif xmax < x:
                    xmax = x

                if ymin > y:
                    ymin = y
                elif ymax < y:
                    ymax = y

            strarray = [[' ']*(xmax-xmin+1) for _ in range(ymax-ymin+1)]

            for c in self:
                x,y = c.vector()
                i = c.type()
                strarray[ymax-y][x-xmin] = tcol(i) + hex(i)

            print "\n",
            for x in strarray:
                print ''.join(x)
            print "\033[0m\n",

        else:
            raise ValueError, 'Ascii plotting implemented only for dimensions 1 and 2.'

    def plot_tikz_boundary(self, color=None, print_text=True, print_tikz_env=True):
        r"""
        """
        s = ''

        if print_tikz_env:
            s += '\\begin{tikzpicture}[x=2.5mm, y=2.5mm]\n'

        if color:
            s += '\\definecolor{cellcolor}{rgb}{%f,%f,%f}\n'%(color[0],color[1],color[2])
            for path in self.boundary():
                s += '\\fill[fill=cellcolor, draw=black]\n'
                s += ' -- '.join(map(str, path[:-1]) + ['cycle;\n'])
        else:
            for path in self.boundary():
                s += '\\draw[black]\n'
                s += ' -- '.join(map(str, path[:-1]) + ['cycle;\n'])

        if print_text:
            for cell in self:
                t = cell.type()
                x, y = cell.vector()
                s += '\\node at (%.1f,%.1f) {\\scriptsize $%s$};\n'%(x+0.5, y+0.5, t)

        if print_tikz_env:
            s += '\\end{tikzpicture}'

        return s

    def plot_occurrences(self, other, color=(0.5,0.5,0.5)):
        r"""
        Plots the occurrences of other in ``self`` with color ``color`` (gray by default).
        """
        G = self.plot()
        other.repaint([color])
        for v in self.occurrences_of(other):
            G += other.translate(v).plot()
        return G

class CombSub(SageObject):
    r"""
    Combinatorial substitutions on Z^d.
    """
    def __init__(self, D, L):
        r"""
        Bla.
        """
        self._base = dict(D)
        rules = []
        patterns = []
        image_patterns = []
        for t1, t2, u, v in L:
            u = tuple(u)
            v = tuple(v)
            rules += [((t1, t2, u), v), ((t2, t1, tuple(-vector(u))), tuple(-vector(v)))]
            patterns.append(Pattern([Cell([0]*len(u),t1,'purple'), Cell(u,t2,'orange')]))
            im1 = self._base[t1]
            im1.repaint(['purple'])
            im2 = self._base[t2].translate(v)
            im2.repaint(['orange'])
            image_patterns.append(im1 + im2)
        self._rules = dict(rules)
        self._patterns = patterns
        self._image_patterns = image_patterns
        self._dimension = len(L[0][2])

    def __iter__(self):
        return iter(self.rules())

    def __repr__(self):
        return 'Set of %d rule(s)'%(len(self._rules)/2)

    def __call__(self, P, iterations=1, patch_union=False, history=False):
        if iterations == 0:
            return Pattern(P)
        elif iterations < 0:
            raise ValueError, "The option 'iterations' must be >= 0."
        if (patch_union or history) and (iterations != 1):
            raise NotImplementedError, "The options 'patch_union' and 'history' implemented only for iterations=1."

        for _ in xrange(iterations):
            if patch_union or history:
                Q = []
            else:
                Q = Pattern([])

            if not P:
                return Q

            G = P.covering_graph(self.patterns())
            if not G.is_connected():
                raise ValueError, "The substitution cannot be applied."

            # DFS on G for placing the images,
            # starting at G.vertices()[0] whose image is placed at 0
            placed = []
            c0 = G.vertices()[0]
            queue = [(c0, tuple([0]*len(c0.vector())))]
            while queue:
                c, u = queue.pop()
                if c not in placed:
                    if patch_union:
                        Q += [self.base(c).translate(u)]
                    elif history:
                        Q += [(c.vector(), c.type())]
                    else:
                        Q += self.base(c).translate(u)
                    placed.append(c)
                    for c2 in G.neighbors(c):
                        if c2 not in placed:
                            u2 = self.rules(c,c2)
                            queue.append((c2, tuple(vector(u)+vector(u2))))
            P = Q
        return Q

    def alphabet(self):
        return self._base.keys()

    def base(self, c):
        P = self._base[c.type()]
        P.repaint([c.color()])
        return P

    def rules(self, c1, c2):
        u = tuple(c2.vector() - c1.vector())
        return self._rules[c1.type(), c2.type(), u]

    def patterns(self):
        return self._patterns

    def image_patterns(self):
        return self._image_patterns

    def plot(self, x=8, y=5, iterations=1, patterns=None):
        G = Graphics()
        vector_prefix = [0]*(len(self._base.values()[0][0].vector()) - 2)

        if patterns is None:
            for i, P in enumerate(reversed(self.patterns())):
                G += (P.translate(vector_prefix+[0,i*y])).plot()
                G += (self(P,iterations).translate(vector_prefix+[x,i*y])).plot()
        else:
            for i, P in enumerate(reversed(patterns)):
                G += (P.translate(vector_prefix+[0,i*y])).plot()
                G += (self(P,iterations).translate(vector_prefix+[x,i*y])).plot()
        return G

    def show(self):
        self.plot().show(aspect_ratio=1, axes=false)

    def plot_tikz_image_by_cell(self, P, iterations=1, ncols=4):
        r"""
            Tikz-prints the concatenated patterns in the image of ``P``
        """
        P = Pattern(P)
        P.repaint('hsv')
        colors = [c.color() for c in P]
        Q = self(P, iterations)
        Limg = [Pattern(Q.cells_of_color(col)) for col in colors]
        Limg = list(set(Limg))
        s = '\\begin{tikzpicture}[x=2.5mm, y=2.5mm]\n'
        for R in Limg:
            s += R.plot_boundary_tikz(print_tikz_env=False)
        s += '\\end{tikzpicture}'
        return s

    def plot_tikz_image_by_color(self, P, iterations=1, ncols=4):
        r"""
            TODO: FIX colors
            Tikz-prints the concatenated patterns in the image of ``P``.
        """
        P = Pattern(P)
        colors = [c.color() for c in P]
        Q = self(P, iterations)
        Limg = [(Pattern(Q.cells_of_color(col)),col) for col in colors]
        Limg = list(set(Limg))
        s = '\\begin{tikzpicture}[x=2.5mm, y=2.5mm]\n'
        for (R,col) in Limg:
            s += R.plot_boundary_tikz(color=col, print_tikz_env=False)
        s += '\\end{tikzpicture}'
        return s

    def plot_tikz(self, iterations=1, ncols=4):
        L = []
        i = 1
        for P in self.patterns():
            P = Pattern(P)
            P0 = Pattern([P[0]])
            P1 = Pattern([P[1]])
            P[0].color((1,1,1))
            P[1].color((0.7,0.7,0.7))
            col0 = P[0].color()
            col1 = P[1].color()
            Q = self(P,iterations)
            Q0 = Pattern(Q.cells_of_color(col0))
            Q1 = Pattern(Q.cells_of_color(col1))
            s = '\\begin{tikzpicture}[x=2.5mm, y=2.5mm]\n'
            s += P0.plot_boundary_tikz(print_tikz_env=False)
            s += P1.plot_boundary_tikz(color=(0.65,0.65,0.65), print_tikz_env=False)
            s += '\\end{tikzpicture}'
            t = '\\begin{tikzpicture}[x=2.5mm, y=2.5mm]\n'
            t += Q0.plot_boundary_tikz(print_tikz_env=False)
            t += Q1.plot_boundary_tikz(color=(0.65,0.65,0.65), print_tikz_env=False)
            t += '\\end{tikzpicture}'
            if i%ncols == 0:
                L.append((s,t,'\\\\\n'))
            elif i%ncols != 0:
                L.append((s,t,'& \quad &\n'))
            i += 1
        s = ''.join(['\\myvcenter{' + s + '}\n& \\mapsto &\n\\myvcenter{' + t + '}\n' + u for s, t, u in L])
        return s

    def plot_base_tikz(self, iterations=0):
        L = [(Pattern([Cell([0,0],i)]), self(P, iterations)) for i, P in self._base.items()]
        s = '  \\qquad\n'.join(['\\myvcenter{' + x.plot_boundary_tikz() + '}\n\\ \\mapsto \\\n\\myvcenter{' + y.plot_boundary_tikz() + '}\n' for x, y in L])
        return s

    def plot_iterations(self, i, P, color={}):
        r"""
        Plots ``i`` iterations of ``self`` on ``P``.
        """
        for _ in range(i):
            P.plot().show(axes=false)
            P.repaint(color)
            P = self(P)
        P.plot().show(axes=false)

    def dimension(self):
        r"""
            Returns the dimension of ``self`` (the dimension of
            the vectors of the cells on which self acts).
        """
        return self._dimension

    def image_vector(self, path):
        r"""
            Returns the image vector of ``path``.
        """
        L = []
        for i in range(len(path) - 1):
            L.append((path[i], path[i+1]))
        if L:
            try:
                return sum([vector(self.rules(x[0], x[1])) for x in L])
            except:
                raise ValueError, "The path is not covered by the starting patterns."
        else:
            return vector([0]*self.dimension())

    def is_consistent(self, certificate=False):
        r"""
            If ``self`` is a 2D domino-complete substitution,
            tests whether it is consistent.
        """
        A = self._base.keys()
        for c1, c2, c3, c4 in itertools.product(A, A, A, A):
            path = [Cell((0,0),c1), Cell((0,1),c2), Cell((1,1),c3), Cell((1,0),c4), Cell((0,0),c1)]
            try:
                if self.image_vector(path):
                    if certificate:
                        return Pattern(path[:-1])
                    else:
                        return False
            except:
                raise NotImplementedError, "Consistency test implemented only for 2D domino-complete substitutions."

        return True

    def is_non_overlapping(self):
        r"""
            If ``self`` is a 2D domino-complete substitution,
            tests whether it is non-overlapping.
            TODO
        """
        return "maybe"

#################################
### Some pre-defined patterns ###
#################################

Ubase = Pattern([Cell([1,1], 1), Cell([0,1], 2), Cell([0,0], 3)])
PU = lambda n : [Pattern([Cell([0,0], k)]) for k in range(1,n+1)]

def LPE(n=3):
    L = []
    for t in tuples(range(1, n + 1), 2):
        L.append(Pattern([Cell([0,0], t[0]), Cell([1,0], t[1])]))
        L.append(Pattern([Cell([0,0], t[0]), Cell([0,1], t[1])]))
    return L

def LPV(n=3):
    L = []
    for t in tuples(range(1, n + 1), 2):
        L.append(Pattern([Cell([0,0], t[0]), Cell([1,1], t[1])]))
        L.append(Pattern([Cell([1,0], t[0]), Cell([0,1], t[1])]))
    return L

def LPEV(n=3):
    return LPE(n) + LPV(n)


##########################
### Some substitutions ###
##########################

# example: overlapping arbitrarily far
Doverlapfar = {1:PU(2)[0], 2:PU(2)[1]}
Roverlapfar1 = (1,1,[1,0],[1,0])
Roverlapfar2 = (1,1,[0,1],[0,1])
Roverlapfar3 = (2,2,[1,0],[1,0])
Roverlapfar4 = (2,2,[0,1],[0,1])
Roverlapfar5 = lambda n : (1,2,[1,0],[n+1,0])
Roverlapfar6 = lambda n : (1,2,[-1,0],[n-1,0])
Roverlapfar7 = lambda n : (1,2,[0,1],[n,1])
Roverlapfar8 = lambda n : (1,2,[0,-1],[n,-1])
Soverlapfar = lambda n : CombSub(Doverlapfar, [Roverlapfar1, Roverlapfar2, Roverlapfar3, Roverlapfar4, Roverlapfar5(n), Roverlapfar6(n), Roverlapfar7(n), Roverlapfar8(n)])
overlaptest = Pattern(matrix(3,[1,1,1,1,1,1,0,0,0,1,1,2,0,0,1]))

# example: Rosema Tijdeman
DRT = {1:LPE(3)[7], 2:LPE(3)[13], 3:PU(3)[0]}
RRT1 = (1,1,[1,0],[-1,-1])
RRT2 = (1,2,[0,1],[1,0])
RRT3 = (1,3,[0,1],[1,0])
RRT4 = (2,2,[1,1],[0,-2])
RRT5 = (2,1,[0,1],[1,-1])
SRT = CombSub(DRT, [RRT1, RRT2, RRT3, RRT4, RRT5])

# example: Hokkaido minimal Pisot
Dhok = {1:LPE(2)[5], 2:PU(5)[2], 3:PU(5)[3], 4:PU(5)[4], 5:PU(5)[0]}
Rhok1 = (1,2,[0,1],[1,0])
Rhok2 = (1,3,[1,0],[-1,0])
Rhok3 = (4,1,[1,0],[-1,-1])
Rhok4 = (2,5,[1,0],[-1,-1])
Rhok5 = (1,3,[1,1],[0,-1])
Rhok6 = (4,1,[0,1],[1,-1])
Rhok7 = (5,2,[1,0],[-1,0])
Rhok8 = (3,1,[1,0],[-1,-1])
Rhok9 = (1,4,[1,1],[0,-1])
Rhok10 = (5,1,[0,1],[1,-1])
Rhok11 = (1,2,[1,0],[-1,0])
Shok = CombSub(Dhok, [Rhok1, Rhok2, Rhok3, Rhok4, Rhok5, Rhok6, Rhok7, Rhok8, Rhok9, Rhok10, Rhok11])

Pperhok = Pattern(matrix(4,[2,5,1,3,5,2,3,1]))
bla0 = Pperhok
bla1 = Pperhok.translate([2,0])
bla2 = Pperhok.translate([4,0])
bla3 = Pperhok.translate([0,4])
bla4 = Pperhok.translate([2,4])
bla5 = Pperhok.translate([4,4])
bla6 = Pperhok.translate([6,4])
bla0.repaint({x:"blue" for x in [1,2,3,4,5]})
bla1.repaint({x:"red" for x in [1,2,3,4,5]})
bla2.repaint({x:"green" for x in [1,2,3,4,5]})
bla3.repaint({x:"yellow" for x in [1,2,3,4,5]})
bla4.repaint({x:"orange" for x in [1,2,3,4,5]})
bla5.repaint({x:"purple" for x in [1,2,3,4,5]})
bla6.repaint({x:"white" for x in [1,2,3,4,5]})
superhok = bla0 + bla1 + bla2 + bla3 + bla4 + bla5 + bla6

# example: 2nd min pisot connexe
Dmini = {1:LPE(3)[7], 2:PU(3)[2], 3:PU(3)[0]}
Rmini1 = (1,2,[0,1],[-1,0])
Rmini2 = (3,1,[1,0],[0,1])
Rmini3 = (1,1,[0,1],[-1,-1])
Rmini4 = (2,1,[1,0],[0,1])
Rmini5 = (3,1,[0,1],[-1,-1])
Smini = CombSub(Dmini, [Rmini1, Rmini2, Rmini3, Rmini4, Rmini5])

# example: 2nd min pisot connected domino-complete variant
Dfini = {1:LPE(3)[3], 2:PU(3)[2], 3:PU(3)[0]}
Rfini1 = (1,1,[1,0],[0,2])
Rfini2 = (1,2,[1,0],[0,2])
Rfini3 = (1,3,[1,0],[0,2])
Rfini4 = (2,1,[1,0],[0,1])
Rfini5 = (2,2,[1,0],[0,1])
Rfini6 = (3,1,[1,0],[0,1])
Rfini7 = (3,2,[1,0],[0,1])
Rfini8 = (3,3,[1,0],[0,1])
Rfini9 = (1,1,[0,1],[-1,-1])
Rfini10 = (1,2,[0,1],[-1,0])
Rfini11 = (1,3,[0,1],[-1,0])
Rfini12 = (2,1,[0,1],[-1,-1])
Rfini13 = (2,2,[0,1],[-1,0])
Rfini14 = (2,3,[0,1],[-1,0])
Rfini15 = (3,2,[0,1],[-1,-1])
Rfini16 = (3,3,[0,1],[-1,-1])
Sfini = CombSub(Dfini, [Rfini1, Rfini2, Rfini3, Rfini4, Rfini5, Rfini6, Rfini7, Rfini8, Rfini9, Rfini10, Rfini11, Rfini12, Rfini13, Rfini14, Rfini15, Rfini16])

# example: Jacobi-Perron
def imgJP3(i,j):
    L = [Cell([0,0],1)]
    for k in range(i):
        L.append(Cell([k,-1],2))
    for k in range(j):
        L.append(Cell([k+1,0],3))
    return Pattern(L)
DJP = lambda i,j : {1:PU(3)[1], 2:PU(3)[2], 3:imgJP3(i,j)}
RJP1 = lambda i : (3, 3, [1,0], [-i,1])
RJP2 = lambda i : (1, 3, [1,0], [-i,1])
RJP3 = lambda j : (2, 3, [1,1], [-j-1,0])
RJP4 = lambda j : (3, 3, [0,-1], [j+1,1])
RJP5 = (1, 3, [-1,-1], [1,0])
RJP6 = (2, 1, [1,0], [0,1])
RJP7 = (2, 3, [0,-1], [1,1])
SJP = lambda i,j : CombSub(DJP(i,j), [RJP3(j), RJP7, RJP2(i), RJP1(i), RJP5, RJP4(j), RJP6])


# example: Thue-Morse
TM1 = Pattern(matrix(2,[1,2,2,1]))
TM2 = Pattern(matrix(2,[2,1,1,2]))
DTM = {1:TM1, 2:TM2}
RTM1 = (1,1,[0,1],[0,2])
RTM2 = (1,1,[1,0],[2,0])
RTM3 = (1,2,[0,1],[0,2])
RTM4 = (1,2,[1,0],[2,0])
RTM5 = (2,1,[0,1],[0,2])
RTM6 = (2,1,[1,0],[2,0])
RTM7 = (2,2,[0,1],[0,2])
RTM8 = (2,2,[1,0],[2,0])
STM = CombSub(DTM, [RTM1, RTM2, RTM3, RTM4, RTM5, RTM6, RTM7, RTM8])

# example: T polyomino
Tquadro = Pattern(matrix(2,[0,1,0,1,1,1]))
DTquadro = {1:Tquadro}
RTquadro1 = (1,1,[1,0],[3,0])
RTquadro2 = (1,1,[0,1],[0,2])
STquadro = CombSub(DTquadro, [RTquadro1, RTquadro2])

# example: L, complete
Lcomp1 = Pattern(matrix(2,[1,0,1,1]))
Lcomp2 = Pattern(matrix(2,[2,0,2,2]))
DLcomp = {1:Lcomp1, 2:Lcomp2}
RLcomp1 = (1,1,[1,0],[2,1])
RLcomp2 = (1,1,[0,1],[0,2])
RLcomp3 = (1,2,[1,0],[2,1])
RLcomp4 = (1,2,[0,1],[0,2])
RLcomp5 = (2,1,[1,0],[2,1])
RLcomp6 = (2,1,[0,1],[0,2])
RLcomp7 = (2,2,[1,0],[2,1])
RLcomp8 = (2,2,[0,1],[0,2])
SLcomp = CombSub(DLcomp, [RLcomp1, RLcomp2, RLcomp3, RLcomp4, RLcomp5, RLcomp6, RLcomp7, RLcomp8])

# example: LGD
LGD = Pattern(matrix(2,[1,0,1,1]))
DLGD = {1:LGD}
RLGD1 = (1,1,[1,0],[1,1])
RLGD2 = (1,1,[0,1],[-1,2])
SLGD = CombSub(DLGD, [RLGD1, RLGD2])

# example: fibo 2*2 Priebe-Frank primer
Pfibo2d1 = Pattern(matrix(2,[2,4,1,3]))
Pfibo2d2 = Pattern(matrix(1,[1,3]))
Pfibo2d3 = Pattern(matrix(2,[2,1]))
Pfibo2d4 = Pattern(matrix(1,[1]))
Dfibo2d = {1:Pfibo2d1, 2:Pfibo2d2, 3:Pfibo2d3, 4:Pfibo2d4}
Rfibo2d1 = (1,1,[0,1],[0,2])
Rfibo2d2 = (1,2,[0,1],[0,2])
Rfibo2d3 = (2,1,[0,1],[0,1])
Rfibo2d4 = (2,2,[0,1],[0,1])
Rfibo2d5 = (3,3,[0,1],[0,2])
Rfibo2d6 = (3,4,[0,1],[0,2])
Rfibo2d7 = (4,3,[0,1],[0,1])
Rfibo2d8 = (4,4,[0,1],[0,1])
Rfibo2d9  = (1,1,[1,0],[2,0])
Rfibo2d10 = (1,3,[1,0],[2,0])
Rfibo2d11 = (3,1,[1,0],[1,0])
Rfibo2d12 = (3,3,[1,0],[1,0])
Rfibo2d13 = (2,2,[1,0],[2,0])
Rfibo2d14 = (2,4,[1,0],[2,0])
Rfibo2d15 = (4,2,[1,0],[1,0])
Rfibo2d16 = (4,4,[1,0],[1,0])
Sfibo2d = CombSub(Dfibo2d, [Rfibo2d1, Rfibo2d2, Rfibo2d3, Rfibo2d4, Rfibo2d5, Rfibo2d6, Rfibo2d7, Rfibo2d8, Rfibo2d9, Rfibo2d10, Rfibo2d11, Rfibo2d12, Rfibo2d13, Rfibo2d14, Rfibo2d15, Rfibo2d16])

# example: dominofun
Pdomfun1 = Pattern(matrix(2,[4,4,3,3]))
Pdomfun2 = Pattern(matrix(2,[1,2,1,2]))
Pdomfun3 = Pattern(matrix(2,[4,4,3,3]))
Pdomfun4 = Pattern(matrix(2,[1,2,1,2]))
Ddomfun = {1:Pdomfun1, 2:Pdomfun2, 3:Pdomfun3, 4:Pdomfun4}
LRdomfun_v = [(i,j,[0,1],[0,2]) for (i,j) in itertools.product(range(1,5), range(1,5))]
LRdomfun_h = [(i,j,[1,0],[2,0]) for (i,j) in itertools.product(range(1,5), range(1,5))]
Sdomfun = CombSub(Ddomfun, LRdomfun_v + LRdomfun_h)

# example: Hafer 3*3, from automatic sequences
PHafer1 = Pattern(matrix(3,[1,1,1,1,1,1,1,1,1])).translate([-1,-1])
PHafer2 = Pattern(matrix(3,[2,1,2,1,2,1,2,1,2])).translate([-1,-1])
DHafer = {1:PHafer2, 2:PHafer1}
RHafer1 = (1,1,[0,1],[0,3])
RHafer2 = (1,1,[1,0],[3,0])
RHafer3 = (1,2,[0,1],[0,3])
RHafer4 = (1,2,[1,0],[3,0])
RHafer5 = (2,1,[0,1],[0,3])
RHafer6 = (2,1,[1,0],[3,0])
RHafer7 = (2,2,[0,1],[0,3])
RHafer8 = (2,2,[1,0],[3,0])
SHafer = CombSub(DHafer, [RHafer1, RHafer2, RHafer3, RHafer4, RHafer5, RHafer6, RHafer7, RHafer8])

# example: 3D triangle
D31 = Pattern([Cell([0,0,0],1), \
               Cell([1,0,0],2), \
               Cell([0,1,0],2), \
               Cell([0,0,1],2)])
D32 = Pattern([Cell([0,0,0],2), \
               Cell([1,0,0],1), \
               Cell([0,1,0],1), \
               Cell([0,0,1],1)])
DD3 ={1:D31, 2:D32}
RD3 = [(i, j, [0,0,1], [0,0,2]) for i, j in tuples([1,2],2)] \
    + [(i, j, [0,1,0], [0,2,0]) for i, j in tuples([1,2],2)] \
    + [(i, j, [1,0,0], [2,0,0]) for i, j in tuples([1,2],2)]
SD3 = CombSub(DD3, RD3)

# example: 3D crosses
cross3D1 = Pattern([Cell([0,0,0],1), \
                    Cell([1,0,0],1), \
                    Cell([0,1,0],2), \
                    Cell([0,0,1],1), \
                    Cell([-1,0,0],2), \
                    Cell([0,-1,0],1), \
                    Cell([0,0,-1],2)])
cross3D2 = Pattern([Cell([0,0,0],2), \
                    Cell([1,0,0],2), \
                    Cell([0,1,0],1), \
                    Cell([0,0,1],2), \
                    Cell([-1,0,0],1), \
                    Cell([0,-1,0],2), \
                    Cell([0,0,-1],1)])
Dcross3D ={1:cross3D1, 2:cross3D2}
Rcross3D = [(i, j, [0,0,1], [0,0,3]) for i, j in tuples([1,2],2)] \
    + [(i, j, [0,1,0], [0,3,0]) for i, j in tuples([1,2],2)] \
    + [(i, j, [1,0,0], [3,0,0]) for i, j in tuples([1,2],2)]
cross3D = CombSub(Dcross3D, Rcross3D)

# example: 3D Menger sponge
patt2_1 = [Pattern([Cell([0,0,0],1), Cell([1,0,0],1)]), \
           Pattern([Cell([0,0,0],1), Cell([0,1,0],1)]), \
           Pattern([Cell([0,0,0],1), Cell([0,0,1],1)])]
Fsponge = Pattern([Cell([-1,-1,-1],1), \
                   Cell([0,-1,-1],1), \
                   Cell([1,-1,-1],1), \
                   Cell([-1,0,-1],1), \
                   Cell([1,0,-1],1), \
                   Cell([-1,1,-1],1), \
                   Cell([0,1,-1],1), \
                   Cell([1,1,-1],1), \
                   Cell([-1,-1,0],1), \
                   Cell([1,-1,0],1), \
                   Cell([-1,1,0],1), \
                   Cell([1,1,0],1), \
                   Cell([-1,-1,1],1), \
                   Cell([0,-1,1],1), \
                   Cell([1,-1,1],1), \
                   Cell([-1,0,1],1), \
                   Cell([1,0,1],1), \
                   Cell([-1,1,1],1), \
                   Cell([0,1,1],1), \
                   Cell([1,1,1],1)])
Dsponge = {1:Fsponge}
Rsponge = [(1,1,(1,0,0),(3,0,0)), (1,1,(0,1,0),(0,3,0)), (1,1,(0,0,1),(0,0,3))]
sponge = CombSub(Dsponge, Rsponge)

